// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.layer.thematicDataDisplayLayer.util;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.*;
import java.text.*;
import java.util.ArrayList;
import java.util.Properties;

import com.bbn.openmap.BasicI18n;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.*;
import fr.unicaen.openmap.dataStorage.*;
import fr.unicaen.openmap.util.Util;
import fr.unicaen.openmap.layer.thematicDataDisplayLayer.gui.ResizerPanel;

/**
 * Utility class to resize a graphical object proportionally to its data value
 * <p>If you want to fix min and max value among different maps 
 * use setMinMaxValue() and successive calls to adjustMinMaxValue(dsi, dataColumnName)
 * instead of calling setMinMaxValue(dsi, dataColumnName).
 * <p>If you precise the valuePerPixel property, it will update maxSize accordingly.
 */
public class Resizer {

    /**
     * GUI for Resizer.
     */
    //protected JComponent panel;
    public static int NO_MAX_SIZE = -1;
    protected JComponent confPanel;
    protected I18n i18n = new BasicI18n();
    /** maxSize corresponds to the size of an element representing a maxValue.
     *  <br>maxSize == valuePerPixel * maxValue 
     */
    double maxSize = 40;
    /** the maximum value */
    private double minValue = 0.0;
    private /** the maximum value */
    double maxValue;
    /** valuePerPixel == maxValue / maxSize; */
    double valuePerPixel = 100;
    /** if true, valuePerPixel is deduced from maxValue and maxSize
     *  otherwise it's valuePerPixel that will set maxSize
     */
    boolean deduceMaxSizeFromValuePerPixel = false;
    /**
     * Specific panel for Resizer.
     */
    protected ResizerPanel defaultGUI;//one by default

    private void syncMaxSize() {
        Debug.message("Resizer", "==> Resizer.syncMaxSize() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        if (getMaxValue() != Double.MIN_VALUE) {
            maxSize = getMaxValue() / valuePerPixel;
        }
        Debug.message("Resizer", "<== Resizer.syncMaxSize() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        fireResizerChanged();
    }

    private void syncValuePerPixel() {
        Debug.message("Resizer", "==> Resizer.syncValuePerPixel() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        if (getMaxValue() != Double.MIN_VALUE) {
            valuePerPixel = getMaxValue() / maxSize;
        }
        Debug.message("Resizer", "<== Resizer.syncValuePerPixel() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        fireResizerChanged();
    }

    /**
     * Returns element's size (width, radius or anything determinating its size)
     * proportionnal to value parameter. 
     */
    public float getSize(double value) {
	return (float) (dc.getValue(value) / valuePerPixel);
    }

    public float getSize(DataObject data) {
        return (float)(dc.getValue(data)/valuePerPixel);
    }

//    /**
//     * Returns element's absolute value proportionnally to the size parameter
//     */
//    public double getValue(double size) {
//	return (double)size / valuePerPixel);
//    }

    protected DataValueComputer dc;
    public Resizer() {
	this(new DataValueComputer());
    }
    public Resizer(DataValueComputer dc) {
	this(dc, NO_MAX_SIZE, 1.0, 0.0);
    }

    public Resizer(double _maxSize, double _maxValue, double _minValue) {
	this(new DataValueComputer(), _maxSize, _maxValue, _minValue);
    }

    public Resizer(DataValueComputer dc, double _maxSize, double _maxValue, double _minValue) {
	this.dc = dc;
        maxSize = _maxSize;
        minValue = _minValue;
        maxValue = _maxValue;
        deduceMaxSizeFromValuePerPixel = false;
        //syncValuePerPixel();
    }

    public void setMinMaxValue(double _minValue, double _maxValue) {
        minValue = _minValue;
        maxValue = _maxValue;
        Debug.message("Resizer", "Resizer.setMinMaxValue(" + _minValue + ", " + _maxValue + ")");
        if (deduceMaxSizeFromValuePerPixel) {
            syncMaxSize();
        } else {
            syncValuePerPixel();
        }
    }

    public void setMinMaxValue() {
        Debug.message("Resizer", "Resizer.setMinMaxValue(infinity, 0)");
        minValue = Double.MAX_VALUE;
        maxValue = Double.MIN_VALUE;
    }

    public void setDeduceMaxSizeFromValuePerPixel(boolean b) {
        deduceMaxSizeFromValuePerPixel = b;
    }

    public boolean getDeduceMaxSizeFromValuePerPixel() {
        return deduceMaxSizeFromValuePerPixel;
    }

    /**
     * Sets the min and max values by computing the min and max values of dataColumnName
     * obtained from the DataStorageIterator 
     */
    public void setMinMaxValue(DataStorageIterator dsi, String dataColumnName) {
        this.setMinMaxValue();
        this.adjustMinMaxValue(dsi, dataColumnName);
    }

    /**
     * Sets the min and max values by computing the min and max values obtained 
     * by applying DataComputer.getValue() to the dataObjects extracted with the
     * DataStorageIterator. 
     */
    protected void setMinMaxValue(DataStorageIterator dsi) {
        this.setMinMaxValue();
        this.adjustMinMaxValue(dsi);
    }

    /**
     * Adjusts the min and max values with the min and max values of dataColumnName
     * obtained throw the DataStorageIterator.
     * <br/>
     * Don't reset min and max as setMinMaxValue method does, making possible to used it incrementaly.
     */
    public void adjustMinMaxValue(DataStorageIterator dsi, String dataColumnName) {
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            double value = Math.abs(Util.parseDouble(data.getData(dataColumnName)));
            if (value < getMinValue()) {
                minValue = value;
            }
            if (value > getMaxValue()) {
                maxValue = value;
            }
        // System.out.println("        adjustMinMaxValue("+dataColumnName+") ["+minValue+","+maxValue+"]"+value);
        }
        dsi.reset();
        this.setMinMaxValue(getMinValue(), getMaxValue());
        Debug.message("Resizer", "Resizer.adjustMinMaxValue(" + dataColumnName + ") [" + getMinValue() + "," + getMaxValue() + "]");
        Debug.message("Resizer", "<= Resizer.adjustMinMaxValue() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
    }

    /**
     * Adjusts the min and max values by computing the min and max values obtained 
     * by applying DataComputer.getValue() to the dataObjects extracted with the
     * DataStorageIterator.
     * <br/>
     * Don't reset min and max as setMinMaxValue method does, making possible to used it incrementaly.
     */
    protected void adjustMinMaxValue(DataStorageIterator dsi) {
        if(maxSize == NO_MAX_SIZE)
            maxSize = getMaxSizeAccordingToNbRecords(dsi.getDataStorage().getCount());
        while (dsi.hasNext()) {
            DataObject data = dsi.next();
            double value = Math.abs(dc.getValue(data));
            if (value < getMinValue()) {
                minValue = value;
            }
            if (value > getMaxValue()) {
                maxValue = value;
            }
	    // System.out.println("        adjustMinMaxValue() ["+minValue+","+maxValue+"]"+value);
        }
        dsi.reset();
        this.setMinMaxValue(getMinValue(), getMaxValue());
        Debug.message("Resizer", "<= "+this+".adjustMinMaxValue(dsi, dc) " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        refreshGUI();
    }

    public void setMaxSize(double _maxSize) {
        if (Double.isInfinite(_maxSize)) {
            return;
        }
        maxSize = _maxSize;
        syncValuePerPixel();
    }

    public void setValuePerPixel(double _valuePerPixel) {
        valuePerPixel = _valuePerPixel;
        syncMaxSize();
    }
    public static final String maxSizePropertyString = "maxSize";
    public static final String valuePerPixelPropertyString = "valuePerPixel";
    public static final String deduceMaxSizeFromValuePerPixelPropertyString = "deduceMaxSizeFromValuePerPixel";

    /**
     * used to set the object with properties ressources
     */
    public void setProperties(String prefix, Properties properties) {
        deduceMaxSizeFromValuePerPixel =
                Util.isTrue(properties.getProperty(prefix + deduceMaxSizeFromValuePerPixelPropertyString, "false"));
        if (deduceMaxSizeFromValuePerPixel) {
            setValuePerPixel(PropUtils.doubleFromProperties(properties, prefix + valuePerPixelPropertyString, 100));
        } else {
            setMaxSize(PropUtils.doubleFromProperties(properties, prefix + maxSizePropertyString, NO_MAX_SIZE));
        }
    }

    public Properties getProperties(String prefix, Properties properties) {
        properties.put(prefix + deduceMaxSizeFromValuePerPixelPropertyString, new Boolean(deduceMaxSizeFromValuePerPixel).toString());
        if (deduceMaxSizeFromValuePerPixel) {
            properties.put(prefix + valuePerPixelPropertyString, String.valueOf(valuePerPixel));
        } else {
            properties.put(prefix + maxSizePropertyString, String.valueOf(maxSize));
        }
        return properties;
    }

    public double getMaxSize() {
        return maxSize;
    }

    public double getValuePerPixel() {
        return valuePerPixel;
    }

    public JComponent getGUI(DataStorageIterator dsi) {

        if (confPanel != null) {
            return confPanel;
        }

        confPanel = new JPanel();
        if(maxSize == NO_MAX_SIZE)
            setMaxSize(getMaxSizeAccordingToNbRecords(dsi.getDataStorage().getCount()));
        defaultGUI = new ResizerPanel(this, true, true, false);
        defaultGUI.setEnabled(true);

        confPanel.add(defaultGUI);

        return confPanel;
    }

    public void refreshGUI() {
        if (defaultGUI != null) {
            defaultGUI.refresh();
        }
    }
    private ArrayList<ResizerListener> listenerList = new ArrayList<ResizerListener>();

    public void addResizerListener(ResizerListener l) {
        listenerList.add(l);
    }

    public void removeResizerListener(ResizerListener l) {
        listenerList.remove(l);
    }

    public void setDataValueComputer(DataValueComputer dc) {
        this.dc = dc;
        fireResizerChanged();
    }

    protected void fireResizerChanged() {
        Debug.message("Resizer", "Resizer.fireResizerChanged() " + getMaxValue() + " " + maxSize + " " + valuePerPixel);
        for (int i = 0; i < listenerList.size(); i++) {
            Debug.message("Resizer", "resizerChanged -> " + i + " " + listenerList.get(i));
            listenerList.get(i).resizerChanged(this);
        }
    }

    public String toString() {
        Properties properties = new Properties();
        return "Resizer@"+hashCode()+" "+getProperties("", properties).toString();
    }

    public int getMaxSizeAccordingToNbRecords(int nbRecords){
        if(nbRecords < 50)
            return 40;
        else
            return 10;

    }

    public DataValueComputer getDataValueComputer(){
        return this.dc;
    }

    /**
     * @return the minValue
     */
    public double getMinValue() {
        return minValue;
    }

    /**
     * @return the maxValue
     */
    public double getMaxValue() {
        return maxValue;
    }
}
