// <header>
/**
 *  Themamap: The Thematic Maps Production Tool
 *
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, Certic
 *  Copyright (c) 2008 Université de Caen Basse-Normandie, GREYC
 *                     		
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  <http://www.gnu.org/licenses/>
 *
 *  See http://themamap.info.unicaen.fr/authors/
 *      for a list of contributors 
 */
// </header>
package fr.unicaen.openmap.util;

import java.awt.Font;
import java.util.Properties;

import com.bbn.openmap.omGraphics.FontSizer;
import com.bbn.openmap.omGraphics.OMText;
import com.bbn.openmap.util.PropUtils;

/**
 * FontSizerShadow provides more accessors and properties. 
 * @see FontSizerPanel
 * @see fr.unicaen.openmap.layer.thematicDataDisplayLayer.visu.Label
 */
public class FontSizerShadow extends FontSizer {

    /**
     * Property name for font name.
     */
    public final static String FONT_PROPERTY_NAME = "font";

    /**
     * Property name for font size.
     */
    public final static String FONT_SIZE_PROPERTY_NAME = "font.size";

    /**
     * Property name for font style.
     */
    public final static String FONT_STYLE_PROPERTY_NAME = "font.style";

    /**
     * Property name for minimum point size.
     */
    public final static String MIN_SIZE_PROPERTY_NAME = "font.min";

    /**
     * Property name for maximum point size.
     */
    public final static String MAX_SIZE_PROPERTY_NAME = "font.max";

    /**
     * Property name for base scale.
     */
    public final static String SCALE_PROPERTY_NAME = "font.scale";

    /**
     * Property name for multiplier.
     */
    public final static String MULTIPLIER_PROPERTY_NAME = "multiplier";	


    
    /**
     * Same as FontSizer.
     * @param font the font to use as the base font.
     * @param baseScale the scale where the base font is shown at its
     *        natural size.
     * @param multiplier to use against the ratio of base
     *        scale:current scale.
     * @param minPointSize the minimum point size to use for the
     *        scaled font.
     * @param maxPointSize the maximum point size to use for the
     *        scaled font.
     */
    public FontSizerShadow(Font font, float baseScale, int multiplier, int minPointSize, int maxPointSize) {
	super(font, baseScale, multiplier, minPointSize, maxPointSize);
    }

    /**
     * Same as Fontsizer.
     * Contructor that uses the OMText DEFAULT_FONT as the base font.
     * 
     * @param baseScale the scale where the base font is shown at its
     *        natural size.
     * @param multiplier to use against the ratio of base
     *        scale:current scale.
     * @param minPointSize the minimum point size to use for the
     *        scaled font.
     * @param maxPointSize the maximum point size to use for the
     *        scaled font.
     */
    public FontSizerShadow(float baseScale, int multiplier, int minPointSize, int maxPointSize) {
	super(baseScale, multiplier, minPointSize, maxPointSize);
    }



    /**
     * Get the base scale used.
     * @return Scale base used 
     */
    public float getBaseScale() {
	return baseScale;
    }

    /**
     * Set the base scale used. The base scale is the scale where the base font is shown as its natural size.
     * In order to refresh, note that Font is automatically scaled.
     * @param baseScale new value for scale
     */
    public void setBaseScale(float baseScale) {
	this.baseScale = baseScale;
	//I need it. If not, scaled font is not updated :\
	setMultiplier(getMultiplier());
    }

    /**
     * Get the minimum point size for scaled font.
     * @return value for minimum point size
     */
    public int getMinPointSize() {
	return minPointSize;
    }
    
    /**
     * Set the minimum point size for scaled font.
     * @param minPointSize new value for the minimum point size
     */
    public void setMinPointSize(int minPointSize) {
	this.minPointSize = minPointSize;
	//I need it. If not, scaled font is not updated :\
	setMultiplier(getMultiplier());
    }
    
    /**
     * Get the maximum point size for scaled font.
     * @return value for maximum point size
     */
    public int getMaxPointSize() {
	return maxPointSize;
    }

    /**
     * Set the maximum point size for scaled font.
     * In order to refresh, note that Font is automatically scaled.
     * @param maxPointSize new value for the maximum point size
     */
    public void setMaxPointSize(int maxPointSize) {
	this.maxPointSize = maxPointSize;
       	//I need it. If not, scaled font is not updated :\
	setMultiplier(getMultiplier());
    }

    /**
     * Used to set FontSizerShadow with properties resources.
     * @param prefix string prefix used in the properties
     * @param properties used to retrieve properties
     */
    public void setProperties(String prefix, Properties properties) {
	setFont(new Font(properties.getProperty(prefix + FONT_PROPERTY_NAME, OMText.DEFAULT_FONT.getName()),
			 PropUtils.intFromProperties(properties, prefix + FONT_STYLE_PROPERTY_NAME, OMText.DEFAULT_FONT.getStyle()),
			 PropUtils.intFromProperties(properties, prefix + FONT_SIZE_PROPERTY_NAME, OMText.DEFAULT_FONT.getSize())
			 )
		);
	setMinPointSize(PropUtils.intFromProperties(properties, prefix + MIN_SIZE_PROPERTY_NAME, getFont().getSize()));
	setMaxPointSize(PropUtils.intFromProperties(properties, prefix + MAX_SIZE_PROPERTY_NAME, getFont().getSize()));
	setBaseScale(PropUtils.floatFromProperties(properties, prefix + SCALE_PROPERTY_NAME, getBaseScale()));
	setMultiplier(PropUtils.intFromProperties(properties, prefix + MULTIPLIER_PROPERTY_NAME, getMultiplier()));
    }

    /**
     * Used to keep properties from FontSizerShadow.
     * @param prefix string prefix used in the properties
     * @param properties used to load properties
     * @return Properties.
     */
    public Properties getProperties(String prefix, Properties properties) {
	//Font
	properties.put(prefix + FONT_PROPERTY_NAME, getFont().getName());
	properties.put(prefix + FONT_STYLE_PROPERTY_NAME, String.valueOf(getFont().getStyle()));
	properties.put(prefix + FONT_SIZE_PROPERTY_NAME, String.valueOf(getFont().getSize()));
	//FontSizer specifics
	properties.put(prefix + MIN_SIZE_PROPERTY_NAME, String.valueOf(minPointSize));
	properties.put(prefix + MAX_SIZE_PROPERTY_NAME, String.valueOf(maxPointSize));
	properties.put(prefix + SCALE_PROPERTY_NAME, String.valueOf(baseScale));
	properties.put(prefix + MULTIPLIER_PROPERTY_NAME, String.valueOf(multiplier));

	return properties;
    }
}
